
/*
 * iconview.cc - code for the main window widget
 *
 * This file is part of the source distribution of 
 * Jafm - an icon based X11 desktop program 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 */


#include <unistd.h>

#include <qdir.h>
#include <qslider.h>
#include <qlined.h>
#include <qpalette.h>
#include <qapp.h>
#include <qmlined.h>
#include <qlist.h>
#include <qregexp.h>
#include <qtstream.h>
#include "iconview.hh"
#include "sfm.hh"
#include "dirbar.hh"

#define INFOBOX_HEIGHT 0
#define CONTROLS_WIDTH 30
#define SCRB_SIZE 20
#define EDGING 4


IconView::IconView (const QDir &dir, Sfm *parent)
  : QWidget (0) 
{

  /* Initiailize a new icon view
   *
   */

  app = parent ;
  connect (app, SIGNAL (dragDropHighlight (WId, const QPoint &, const QList<DeskIcon> &)),
	   this, SLOT (dragDropHighlight (WId, const QPoint &, const QList<DeskIcon> &)) ) ; 
  connect (app, SIGNAL (drag_drop_happened (WId, const QPoint &, const QList<DeskIcon> &)),
	   this, SLOT (drag_drop_happened (WId, const QPoint &, const QList<DeskIcon> &)) ) ;

  // divide up into areas
  
  // init menus ;
  mb = new MainMenuBar (this) ;
  mb->resize (width (), 20) ;
  connect (mb, SIGNAL (openDocument()), this, SLOT(openIcon()) ) ;
  connect (mb, SIGNAL (closeThis()), this, SLOT (close ()) ) ;
  connect (mb, SIGNAL (newApplWin ()), this, SLOT (openAppl ())) ;
  connect (mb, SIGNAL (quitApp ()), (const QObject *)parent, SLOT ( quit () ) ) ;

  // init directory bar
  db = new DirectoryBar (dir, this) ;
  db-> setGeometry (0, mb->height(), width(), 40) ;
  heightOffset = db->height () + 5 ;
  
  // space for slider etc controls
  controls = new QWidget (this) ;
  controls->setGeometry (0, heightOffset+mb->height(), CONTROLS_WIDTH, height() - (heightOffset+mb->height())) ;

  // info text space
  // infobox = new QMultiLineEdit (this) ;
  // infobox->setGeometry (0, height() - INFOBOX_HEIGHT, width(),INFOBOX_HEIGHT) ;
  // infobox->setReadOnly (true) ;

  rootDesk = parent->desktop () ;
  desk = new IconDesk (this) ;
  desk->move( CONTROLS_WIDTH, mb->height()+heightOffset) ;
  
  desk->setPalette ( QPalette ( QColor (120, 171, 172)) ) ;		    

  // allow the desk to communicate draggin 'n droppin
  connect (desk, SIGNAL (dragPointAt(const QPoint &, const QList<DeskIcon> &)), app,
	   SLOT (dragPointAt (const QPoint &, const QList<DeskIcon> &)) ) ;
  connect (desk, SIGNAL (dragDropRelease (const QPoint &, const QList<DeskIcon> &)),
	   app, SLOT (dragDropRelease (const QPoint &, const QList<DeskIcon> &)) ) ;

  connect (desk, SIGNAL (newDirWindow (const char *)),
	    app, SLOT (open (const char *))) ;
  
  connect (desk, SIGNAL (signal_refresh ()),
	   app, SLOT (slot_refresh ())) ;

  // init scroll bars
  hscrBar = new QScrollBar (QScrollBar::Horizontal, this) ;
  hscrBar-> setGeometry (50, height()-20, width () - 50, 20) ;
  vscrBar = new QScrollBar (QScrollBar::Vertical, this) ;
  vscrBar-> setGeometry (width()-SCRB_SIZE, heightOffset+mb->height(), SCRB_SIZE, height() - (heightOffset+mb->height()+INFOBOX_HEIGHT+hscrBar->height())) ;

  desk->resize (width()-SCRB_SIZE-CONTROLS_WIDTH, height()-(heightOffset+mb->height()+INFOBOX_HEIGHT+hscrBar->height())) ;

  connect (hscrBar, SIGNAL (valueChanged (int)),
	   desk, SLOT (horizScroll (int))) ;
  connect (vscrBar, SIGNAL (valueChanged (int)),
	   desk, SLOT (verticScroll (int))) ;


  connect (desk, SIGNAL (resized (int, int)),
	   this, SLOT (deskResized (int, int))) ;


  // init scale slider
  scaler = new QSlider (1,20, 1, 6, QSlider::Vertical, this) ;
  scaler->setStyle (WindowsStyle) ; // better than motif
  scaler->setPalette (*QApplication::palette()) ;
  scaler->setGeometry (5,120,20,150) ;
  connect (scaler, SIGNAL (valueChanged (int)),
	   desk, SLOT (rescale (int)) ) ;
  currentDir = dir ;

  connect (db, SIGNAL (dirChanged (const QDir &)),
	   this, SLOT (changeDir (const QDir &)) ) ;
  desk->read (dir) ;

}

IconView::IconView (const char *c,  Sfm *parent)
{
  
  /* Initialize a new icon view
   * where the icons are not files
   */

  app = parent ;
  setCaption ("Applications") ;
  app = parent ;
  connect (app, SIGNAL (dragDropHighlight (WId, const QPoint &, const QList<DeskIcon> &)),
	   this, SLOT (dragDropHighlight (WId, const QPoint &, const QList<DeskIcon> &)) ) ; 
  connect (app, SIGNAL (drag_drop_happened (WId, const QPoint &, const QList<DeskIcon> &)),
	   this, SLOT (drag_drop_happened (WId, const QPoint &, const QList<DeskIcon> &)) ) ;

  // init menus ;
  mb = new MainMenuBar (this) ;
  mb->resize (width (), 20) ;
  connect (mb, SIGNAL (openDocument()), this, SLOT(openIcon()) ) ;
  connect (mb, SIGNAL (closeThis()), this, SLOT (close ()) ) ;
  connect (mb, SIGNAL (quitApp ()), (const QObject *)parent, SLOT ( quit () ) ) ;

  // no directory bar
  db = 0 ;
  heightOffset = 35 ;

  // space for slider etc controls
  controls = new QWidget (this) ;
  controls->setGeometry (0, heightOffset+mb->height(), CONTROLS_WIDTH, height() - (heightOffset+mb->height())) ;

  // info text space
  //  infobox = new QMultiLineEdit (this) ;
  // infobox->setGeometry (0, height() - INFOBOX_HEIGHT, width(),INFOBOX_HEIGHT) ;
  // infobox->setReadOnly (true) ;

  rootDesk = parent->desktop () ;
  desk = new IconDesk (this) ;
  desk->move( CONTROLS_WIDTH, mb->height()+heightOffset) ;
  
  desk->setPalette ( QPalette ( QColor (100, 150, 250)) ) ;		    

  // allow the desk to communicate draggin 'n droppin
  connect (desk, SIGNAL (dragPointAt(const QPoint &, const QList<DeskIcon> &)), app,
	   SLOT (dragPointAt (const QPoint &, const QList<DeskIcon> &)) ) ;
  connect (desk, SIGNAL (dragDropRelease (const QPoint &, const QList<DeskIcon> &)),
	   app, SLOT (dragDropRelease (const QPoint &, const QList<DeskIcon> &)) ) ;

  // init scroll bars
  hscrBar = new QScrollBar (QScrollBar::Horizontal, this) ;
  hscrBar-> setGeometry (50, height()-20, width () - 50, 20) ;
  vscrBar = new QScrollBar (QScrollBar::Vertical, this) ;
  vscrBar-> setGeometry (width()-SCRB_SIZE, heightOffset+mb->height(), SCRB_SIZE, height() - (heightOffset+mb->height()+INFOBOX_HEIGHT+hscrBar->height())) ;

  desk->resize (width()-SCRB_SIZE-CONTROLS_WIDTH, height()-(heightOffset+mb->height()+INFOBOX_HEIGHT+hscrBar->height())) ;

  connect (hscrBar, SIGNAL (valueChanged (int)),
	   desk, SLOT (horizScroll (int))) ;
  connect (vscrBar, SIGNAL (valueChanged (int)),
	   desk, SLOT (verticScroll (int))) ;


  connect (desk, SIGNAL (resized (int, int)),
	   this, SLOT (deskResized (int, int))) ;


  // init scale slider
  scaler = new QSlider (1,20, 1, 6, QSlider::Vertical, this) ;
  scaler->setGeometry (5,120,20,150) ;
  scaler->setStyle (WindowsStyle) ; // better than motif
  scaler->setPalette (*QApplication::palette()) ;
  connect (scaler, SIGNAL (valueChanged (int)),
	   desk, SLOT (rescale (int)) ) ;
  currentDir = 0 ;

  //  connect (db, SIGNAL (dirChanged (const QDir &)),
  // this, SLOT (changeDir (const QDir &)) ) ;
  desk->read (c) ;
}

void IconView::close ()
{
  app->close (this) ;
}

void IconView::deskResized (int w, int h)
{
  hscrBar->setRange (0, 
		     w - desk->width () >0 ? (w - desk->width ())/50 + 1: 0) ;
  vscrBar->setRange (0, 
		     h - desk->height () > 0 ?  (h - desk->height ())/50 + 2 : 0) ;
}

void IconView::changeDir (const char *c)
{
  changeDir (QDir (c)) ;
}


void IconView::changeDir (const QDir &dir) 
{
  /* 
   * change directory to specified
   */ 
   
  currentDir = dir ;
  setCaption (dir.dirName()) ;setIconText (dir.dirName()) ;
  if (!QDir::setCurrent ((const char *)dir.absPath()) ) {
    debug ("couldn't set current directory to %s",(const char *)(dir.absPath())) ;
  }
  desk->read (dir) ;
  //  db->reset (dir) ;
}

void IconView::paintEvent (QPaintEvent *pe)
{
  (void) pe ;
  QPainter p ;
  p.begin (this) ;

  //  p.drawLine (0,78,width (), 78) ;
  // p.drawLine (0,80,width (), 80) ;

  //draw each of the widgets
  
  p.end () ;

  
}

void IconView::resizeEvent (QResizeEvent *)
{
  mb->resize (width (), 0) ;
  if (db) db->resize (width ()-20, 40) ;
  controls->setGeometry (0, heightOffset+mb->height(), CONTROLS_WIDTH, 
			 height() - (heightOffset+mb->height())) ;
  hscrBar-> setGeometry (CONTROLS_WIDTH, height()-SCRB_SIZE-INFOBOX_HEIGHT-EDGING,
			 width () - CONTROLS_WIDTH - SCRB_SIZE-EDGING, SCRB_SIZE) ;
  vscrBar-> setGeometry (width()-SCRB_SIZE-EDGING, heightOffset+mb->height(),
			 SCRB_SIZE, height() - 
			 (heightOffset+mb->height()+INFOBOX_HEIGHT+hscrBar->height()+EDGING)) ;
  desk->resize (width()-SCRB_SIZE-CONTROLS_WIDTH-EDGING, 
		height()-(heightOffset+mb->height()+INFOBOX_HEIGHT+hscrBar->height()+EDGING)) ;
  //  infobox->setGeometry (0, height() - INFOBOX_HEIGHT, width(),INFOBOX_HEIGHT) ; 
}

void IconView::openIcon (const QList<Icon> *icList, QMouseEvent *me=0)
{
  /* the user has indicated that the icon(s) should be 
   * 'opened', or whatever should naturally be done to
   * it. 
   */
  (void) me ;
  
  QListIterator<Icon> it (*icList) ;

  for ( ; it.current() ; ++it) {
    
    /*    const QFileInfo *finfo = it.current()->get_file_info () ;
    if (finfo == 0)
      continue ;
      */
    
    if (it.current()->is_dir()) {
      //      emit newDirWindow (it.current()->get_full_name ()) ;
      app->open (it.current()->get_full_name ()) ;
    } else {
      it.current()->open_action () ;
    }

    // rest of this is old code
    /*
    // is it a directory?
    if (finfo->isDir()) {
      
      if (finfo->isReadable() && finfo->isExecutable()) {
	if (! (strcmp (finfo->fileName(), "."))) {
	  return ; // it's just us!
	} else {
	  if (me->state() & MidButton) {
	    //	    app->open (finfo.absFilePath ()) ;
	  } else {
	    //	    db->reset (finfo.absFilePath ()) ;
	  }
	}
      }
    } else {
      debug ("doubleclicked %s",(const char *) finfo->absFilePath () ) ;
      //char *execstr = Icon::matchToRules (finfo, Icon::openaction) ;
      char *execstr = 0 ;
      if (execstr) {
	
	QString *qexecstr = new QString (execstr) ;
	// replace occurrences of % with the filename
	qexecstr->replace (QRegExp ("%"), finfo->fileName()) ;
	qexecstr->insert (0, '"') ;
	(*qexecstr) += ' ' ;
	qexecstr->append ( (const char *)(finfo->fileName())) ;
	(*qexecstr) += '"' ;
	debug ("doing %s",(const char *)(*qexecstr)) ;
	// fork and execute 
	
	if (!fork ()) { //child
	  
	  // build up an arg list
	  
	  
	  execlp (execstr,execstr, (const char *)(finfo->fileName()),  0 ) ;
	  perror ("exec failed") ;
	  exit (1) ;
	}
	
	// parent, do nothing
      }
    }
    */
  }
}

void IconView::openAppl ()
{
  app->openAppl ("Applications") ;
}

void IconView::dragDropHighlight (WId h, const QPoint &pos, const QList<DeskIcon> &) 
{
  if (h == winId()) {
    // it's us

    QPoint p = mapFromGlobal (pos) ;
    
    if (desk) {
      if (desk->geometry().contains(p)) {
	desk->checkDragDrop (p) ;
	if (db) db->unHighlightDragDrop() ;	
	return ;
      }
    } 
    
    if (db) {
      if (db->geometry ().contains (p)) {     
	// only if it is exactly one directory!
	// how?
	db->indicateDragDrop (p) ;
	if (desk) desk->unHighlightDragDrop () ;
	return ;
      }
    }
  }
    
  if (db) db->unHighlightDragDrop() ;
  if (desk) desk->unHighlightDragDrop () ;
}

void IconView::drag_drop_happened (WId theWinId, const QPoint &pos, const QList<DeskIcon> &icList)
{
  /*
   * some icon(s) has been dropped on our window
   * at point p (in global coords)
   */
  if (winId() != theWinId)
    return ;

  QPoint p = mapFromGlobal (pos) ;
  
  if (desk) {
    if (desk->geometry().contains(p)) {
      desk->drag_drop_happened (p, icList) ;
      //debug ("In my desk") ;
      return ;
    }
  }
  // dirbar ?
  if (db) {
    if (db->geometry ().contains (p)) {
      if (icList.count() == 1) {
	db->drag_drop_happened ((const char *) icList.getFirst()->get_icon()->get_full_name()) ;
      }
      //debug ("in my dirbar") ;
    }
  }
}

void IconView::openIcon ()
{
  /* overloaded function:
   * call openIcon with the selectedList of the desktop
   */
  
  openIcon (desk->getSelectedList()) ;
}


void IconView::refresh ()
{
  desk->slot_refresh() ;
}



/* ----------------------------------------------------------------------------
 *
 * save-restore state functions
 *
 */

QTextStream & operator << (QTextStream &s, const IconView &v)
{
  // we just record geometry and directory
  s << v.x()  << ' ' << v.y() << ' ' << v.width() << ' ' << v.height() << '\n' ;
  s << (const char *) v.getCurrentDir() << '\n' ;
  return s ;
}



